import Flutter
import UIKit
import PinchSDK
import PureSDK

public class SwiftPinchPlugin: NSObject, FlutterPlugin {
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "com.fluxloop.pinch/sdk", binaryMessenger: registrar.messenger())
    let instance = SwiftPinchPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
      let method: String = call.method
      
      if (method == "start") {
          start(result)
      } else if (method == "stop") {
          stop(result)
      } else if (method == "getStarted") {
          getStarted(result)
      } else if (method == "addCustomEvent") {
          addCustomEvent(result, with: call.arguments)
      } else if (method == "setMetadata") {
          setMetadata(result, with: call.arguments)
      } else if (method == "sendDemographicProfile") {
          sendDemographicProfile(result, with: call.arguments)
      } else {
          result(FlutterMethodNotImplemented)
      }
  }
  
  private func sendDemographicProfile(_ result: @escaping FlutterResult, with arguments: Any?) {
      guard let args = arguments as? [String: Any] else {
          result(FlutterError(code: "INVALID_ARG", message: "Invalid arguments supplied", details: nil))
          return
      }
      
      if let birthYear = args["birthYear"] as? Int, let gender = args["gender"] as? String {
          var dict = Dictionary<String, Any>()
          dict["birthYear"] = birthYear
          dict["gender"] = genderToInt(gender: gender)
          
          Pinch.setMetadata(type: "DemographicProfile", json: dict) { (success) in
              result(success)
          }
      } else {
          result(FlutterError(code: "INVALID_CONTENT", message: "Invalid content supplied", details: nil))
      }
  }

  
  private func setMetadata(_ result: @escaping FlutterResult, with arguments: Any?) {
      guard let args = arguments as? [String: Any] else {
          result(FlutterError(code: "INVALID_ARG", message: "Invalid arguments supplied", details: nil))
          return
      }
      
      if let json = args["json"] as? String, let type = args["type"] as? String {
          if let dict = (convertToDictionary(json: json)) {
              Pinch.setMetadata(type: type, json: dict) { (success) in
                  result(success)
              }
          } else {
              result(FlutterError(code: "CAST_ERR", message: "Failed to cast arguments to [String: Any]", details: nil))
          }
      } else {
          result(FlutterError(code: "INVALID_CONTENT", message: "Invalid content supplied", details: nil))
      }
  }
  
  
  private func addCustomEvent(_ result: @escaping FlutterResult, with arguments: Any?) {
      guard let args = arguments as? [String: Any] else {
          result(FlutterError(code: "INVALID_ARG", message: "Invalid arguments supplied", details: nil))
          return
      }
      
      if let json = args["json"] as? String, let type = args["type"] as? String {
          if let dict = (convertToDictionary(json: json)) {
              Pinch.addCustomData(type: type, json: dict) { (success) in
                  result(success)
              }
          } else {
              result(FlutterError(code: "CAST_ERR", message: "Failed to cast arguments to [String: Any]", details: nil))
          }
      } else {
          result(FlutterError(code: "INVALID_CONTENT", message: "Invalid content supplied", details: nil))
      }
  }
  
  
  private func start(_ result: @escaping FlutterResult) {
      Pinch.start()
      result(true)
  }
  
  private func stop(_ result: @escaping FlutterResult) {
      Pinch.stop()
      result(false)
  }
  
  private func getStarted(_ result: @escaping FlutterResult) {
      result(Pinch.started)
  }
  

  
  
  
  private func convertToDictionary(json: String) -> [String: Any]? {
      if let data = json.data(using: .utf8) {
          do {
              return try JSONSerialization.jsonObject(with: data, options: []) as? [String: Any]
          } catch {
              //
          }
      }
      return nil
  }
  
  private func genderToInt(gender: String) -> Int {
      switch gender {
      case "male":
          return 1
      case "female":
          return 2
      default:
          return 0
      }
  }
}
